@@ -23,9 +23,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 
-#include "libpq-fe.h"
-#include "c.h"
-
 #include "slon.h"
 
 
@@ -40,7 +37,7 @@
  * Static data 
  * ----------
  */
-static int	sched_status = SCHED_STATUS_OK;
+static ScheduleStatus	sched_status = SCHED_STATUS_OK;
 
 static int	sched_numfd = 0;
 static fd_set sched_fdset_read;
@@ -59,7 +56,7 @@ static pthread_cond_t sched_master_cond;
  * Local functions 
  * ----------
  */
-static void *sched_mainloop(void *);
+static void *sched_mainloop(void* );
 static void sched_add_fdset(int fd, fd_set * fds);
 static void sched_remove_fdset(int fd, fd_set * fds);
 /* static void sched_shutdown(); */
@@ -75,7 +72,7 @@ static void sched_remove_fdset(int fd, fd_set * fds);
  * ----------
  */
 int
-sched_start_mainloop(void)
+sched_start_mainloop(void )
 {
 	sched_status = SCHED_STATUS_OK;
 	sched_waitqueue_head = NULL;
@@ -193,8 +190,8 @@ sched_wait_mainloop(void)
 int
 sched_wait_conn(SlonConn * conn, int condition)
 {
-	int			rc;
-
+	ScheduleStatus			rc;
+	int                     fds;
 	/*
 	 * Grab the master lock and check that we're in normal runmode
 	 */
@@ -211,9 +208,15 @@ sched_wait_conn(SlonConn * conn, int condition)
 	 */
 	conn->condition = condition;
 	if (condition & SCHED_WAIT_SOCK_READ)
-		sched_add_fdset(PQsocket(conn->dbconn), &sched_fdset_read);
+	{
+		fds = PQsocket(conn->dbconn);
+		sched_add_fdset(fds, &sched_fdset_read);
+	}
 	if (condition & SCHED_WAIT_SOCK_WRITE)
-		sched_add_fdset(PQsocket(conn->dbconn), &sched_fdset_write);
+	{
+		fds = PQsocket(conn->dbconn);
+		sched_add_fdset(fds, &sched_fdset_write);
+	}
 
 	/*
 	 * Add the connection to the wait queue
@@ -316,7 +319,7 @@ sched_msleep(SlonNode * node, int msec)
 int
 sched_get_status(void)
 {
-	int			status;
+	ScheduleStatus			status;
 
 	pthread_mutex_lock(&sched_master_lock);
 	status = sched_status;
@@ -385,7 +388,7 @@ sched_wakeup_node(int no_id)
  * ----------
  */
 static void *
-sched_mainloop(void *dummy)
+sched_mainloop(void * dummy)
 {
 	fd_set		rfds;
 	fd_set		wfds;
@@ -573,9 +576,12 @@ sched_mainloop(void *dummy)
 		conn = sched_waitqueue_head;
 		while (rc > 0 && conn)
 		{
+			int fd_check = PQsocket(conn->dbconn);
 			if (conn->condition & SCHED_WAIT_SOCK_READ)
 			{
-				if (FD_ISSET(PQsocket(conn->dbconn), &rfds))
+
+				
+				if (fd_check >= 0 && FD_ISSET(fd_check, &rfds))
 				{
 					next = conn->next;
 
@@ -600,7 +606,7 @@ sched_mainloop(void *dummy)
 			}
 			if (conn->condition & SCHED_WAIT_SOCK_WRITE)
 			{
-				if (FD_ISSET(PQsocket(conn->dbconn), &wfds))
+				if (fd_check >= 0 && FD_ISSET(fd_check, &wfds))
 				{
 					next = conn->next;
 
@@ -681,9 +687,12 @@ sched_mainloop(void *dummy)
 static void
 sched_add_fdset(int fd, fd_set * fds)
 {
-	FD_SET(fd, fds);
-	if (fd >= sched_numfd)
-		sched_numfd = fd + 1;
+	if( fd >= 0 && fds != NULL )
+	{
+		FD_SET(fd, fds);
+		if (fd >= sched_numfd)
+			sched_numfd = fd + 1;
+	}
 }
 
 
@@ -697,16 +706,19 @@ sched_add_fdset(int fd, fd_set * fds)
 static void
 sched_remove_fdset(int fd, fd_set * fds)
 {
-	FD_CLR(fd, fds);
-	if (sched_numfd == (fd + 1))
+	if(fd >= 0)
 	{
-		while (sched_numfd > 0)
+		FD_CLR(fd, fds);
+		if (sched_numfd == (fd + 1))
 		{
-			if (FD_ISSET(sched_numfd - 1, &sched_fdset_read))
-				break;
-			if (FD_ISSET(sched_numfd - 1, &sched_fdset_write))
-				break;
-			sched_numfd--;
+			while (sched_numfd > 0)
+			{
+				if (FD_ISSET(sched_numfd - 1, &sched_fdset_read))
+					break;
+				if (FD_ISSET(sched_numfd - 1, &sched_fdset_write))
+					break;
+				sched_numfd--;
+			}
 		}
 	}
 }
