@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slon.c,v 1.79.2.1 2009-08-17 17:09:58 devrim Exp $
+ *	
  *-------------------------------------------------------------------------
  */
 
@@ -20,10 +20,13 @@
 #include <string.h>
 #include <errno.h>
 #include <signal.h>
+#ifndef WIN32
 #include <sys/time.h>
+#endif
 #include <sys/types.h>
 #include <sys/wait.h>
 
+
 #ifdef WIN32
 #include <winsock2.h>
 #include "port/win32service.h"
@@ -53,6 +56,7 @@ int			sched_wakeuppipe[2];
 
 pthread_mutex_t slon_wait_listen_lock;
 pthread_cond_t slon_wait_listen_cond;
+int            slon_listen_started=0;
 
 /* ---------- 
  * Local data 
@@ -77,12 +81,19 @@ static void SlonWatchdog(void);
 static void sighandler(int signo);
 void slon_terminate_worker(void);
 #endif
+typedef void (*sighandler_t)(int);
+static sighandler_t install_signal_handler(int signum, sighandler_t  handler);
 
 int			slon_log_level;
 char	   *pid_file;
 char	   *archive_dir = NULL;
 static int			child_status;
 
+/**
+ * A variable to indicate that the 
+ * worker has been restarted by the watchdog.
+ */
+int worker_restarted=0;
 
 /* ----------
  * Usage
@@ -346,6 +357,18 @@ main(int argc, char *const argv[])
 		slon_exit(-1);
 	}
 
+	if (!PQisthreadsafe()) 
+	{
+		slon_log(SLON_FATAL,"slon: libpq was not compiled with thread safety enabled (normally: --enable-thread-safety).  slon is a multithreaded application requiring thread-safe libpq\n");
+		slon_exit(-1);
+	}
+
+	if (!PQisthreadsafe()) 
+	{
+		slon_log(SLON_FATAL,"slon: libpq was not compiled with --enable-thread-safety. Slony-I requires a thread enabled libpq\n");
+		slon_exit(-1);
+	}
+
 	/*
 	 * There is no watchdog process on win32. We delegate restarting and other
 	 * such tasks to the Service Control Manager. And win32 doesn't support
@@ -695,6 +718,14 @@ SlonMain(void)
 		slon_retry();
 	}
 	pthread_cond_wait(&slon_wait_listen_cond, &slon_wait_listen_lock);
+	if(!slon_listen_started)
+	{
+		/**
+		 * The local listen thread did not start up properly.
+		 */
+		slon_log(SLON_FATAL,"main: localListenThread did not start\n");
+		slon_abort();
+	}
 	pthread_mutex_unlock(&slon_wait_listen_lock);
 
 	/*
@@ -787,59 +818,12 @@ static void
 SlonWatchdog(void)
 {
 	pid_t		pid;
-
-#if !defined(CYGWIN) && !defined(WIN32)
-	struct sigaction act;
-#endif
+	int shutdown=0;
+	int return_code=-99;
+	char * termination_reason="unknown";
 	slon_log(SLON_INFO, "slon: watchdog process started\n");
 
-	/*
-	 * Install signal handlers
-	 */
-#ifndef CYGWIN
-	act.sa_handler = &sighandler;
-	(void) sigemptyset(&act.sa_mask);
-	act.sa_flags = SA_NODEFER;
 
-	if (sigaction(SIGHUP, &act, NULL) < 0)
-#else
-	if (signal(SIGHUP, sighandler) == SIG_ERR)
-#endif
-	{
-		slon_log(SLON_FATAL, "slon: SIGHUP signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
-
-	if (signal(SIGUSR1, sighandler) == SIG_ERR)
-	{
-		slon_log(SLON_FATAL, "slon: SIGUSR1 signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
-	if (signal(SIGALRM, sighandler) == SIG_ERR)
-	{
-		slon_log(SLON_FATAL, "slon: SIGALRM signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
-	if (signal(SIGINT, sighandler) == SIG_ERR)
-	{
-		slon_log(SLON_FATAL, "slon: SIGINT signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
-	if (signal(SIGTERM, sighandler) == SIG_ERR)
-	{
-		slon_log(SLON_FATAL, "slon: SIGTERM signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
-	if (signal(SIGCHLD, sighandler) == SIG_ERR)
-	{
-		slon_log(SLON_FATAL, "slon: SIGCHLD signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
-	if (signal(SIGQUIT, sighandler) == SIG_ERR)
-	{
-		slon_log(SLON_FATAL, "slon: SIGQUIT signal handler setup failed -(%d) %s\n", errno, strerror(errno));
-		slon_exit(-1);
-	}
 
 	slon_log(SLON_CONFIG, "slon: watchdog ready - pid = %d\n", slon_watchdog_pid);
 
@@ -849,53 +833,147 @@ SlonWatchdog(void)
 		SlonMain();
 		exit(-1);
 	}
+	else if (slon_worker_pid < 0) 
+	{
+		slon_log(SLON_FATAL, "slon: failed to fork child: %d %s\n",
+				 errno,strerror(errno));
+		slon_exit(-1);
+		
+	}
+		/*
+	 * Install signal handlers
+	 */
+
+	if (install_signal_handler(SIGHUP, sighandler) == SIG_ERR)
+	{
+		slon_log(SLON_FATAL, "slon: SIGHUP signal handler setup failed -(%d) %s\n", errno, strerror(errno));
+		slon_exit(-1);
+	}
+
+	if (install_signal_handler(SIGUSR1,sighandler) == SIG_ERR)
+	{
+		slon_log(SLON_FATAL, "slon: SIGUSR1 signal handler setup failed -(%d) %s\n", errno, strerror(errno));
+		slon_exit(-1);
+	}
+	if (install_signal_handler(SIGALRM,sighandler) == SIG_ERR)
+	{
+		slon_log(SLON_FATAL, "slon: SIGALRM signal handler setup failed -(%d) %s\n", errno, strerror(errno));
+		slon_exit(-1);
+	}
+	if (install_signal_handler(SIGINT,sighandler) == SIG_ERR)
+	{
+		slon_log(SLON_FATAL, "slon: SIGINT signal handler setup failed -(%d) %s\n", errno, strerror(errno));
+		slon_exit(-1);
+	}
+	if (install_signal_handler(SIGTERM,sighandler) == SIG_ERR)
+	{
+		slon_log(SLON_FATAL, "slon: SIGTERM signal handler setup failed -(%d) %s\n", errno, strerror(errno));
+		slon_exit(-1);
+	}
+
+
+	if (install_signal_handler(SIGQUIT,sighandler) == SIG_ERR)
+	{
+		slon_log(SLON_FATAL, "slon: SIGQUIT signal handler setup failed -(%d) %s\n", errno, strerror(errno));
+		slon_exit(-1);
+	}
 
 	slon_log(SLON_CONFIG, "slon: worker process created - pid = %d\n",
 			 slon_worker_pid);
-	while ((pid = wait(&child_status)) != slon_worker_pid)
+	while(!shutdown)
 	{
-		if (pid < 0 && errno == EINTR)
-			continue;
+		while ((pid = wait(&child_status)) != slon_worker_pid)
+		{
+			if (pid < 0 && errno == EINTR)
+				continue;			
 
-		slon_log(SLON_CONFIG, "slon: child terminated status: %d; pid: %d, current worker pid: %d\n", child_status, pid, slon_worker_pid);
-	}
-	slon_log(SLON_CONFIG, "slon: child terminated status: %d; pid: %d, current worker pid: %d\n", child_status, pid, slon_worker_pid);
-
-	(void) alarm(0);
-
-	switch (watchdog_status)
-	{
-		case SLON_WATCHDOG_RESTART:
-			(void) execvp(main_argv[0], main_argv);
-			slon_log(SLON_FATAL, "slon: cannot restart via execvp() - %s\n",
-					 strerror(errno));
-			slon_exit(-1);
-			break;
-
-		case SLON_WATCHDOG_NORMAL:
-		case SLON_WATCHDOG_RETRY:
-			watchdog_status = SLON_WATCHDOG_RETRY;
-			if (child_status != 0)
+			slon_log(SLON_CONFIG, "slon: child terminated status: %d; pid: %d, current worker pid: %d errno: %d\n", child_status, pid, slon_worker_pid,errno);
+		
+			if(pid < 0 )
 			{
-				slon_log(SLON_CONFIG, "slon: restart of worker in 10 seconds\n");
-				(void) sleep(10);
+				/**
+				 * if errno is not EINTR and pid<0 we have
+				 * a problem.
+				 * looping on wait() isn't a good idea.
+				 */
+				slon_log(SLON_FATAL,"slon: wait returned an error pid:%d errno:%d\n",
+						 pid,errno);
+				exit(-1);
 			}
-			else
-			{
-				slon_log(SLON_CONFIG, "slon: restart of worker\n");
-			}
-			if (watchdog_status == SLON_WATCHDOG_RETRY)
-			{
-				(void) execvp(main_argv[0], main_argv);
-				slon_log(SLON_FATAL, "slon: cannot restart via execvp() - %s\n",
-						 strerror(errno));
-				slon_exit(-1);
-			}
-			break;
+		}
+		if( WIFSIGNALED(child_status) ) 
+		{
+			return_code=WTERMSIG(child_status);
+			termination_reason="signal";	
+		}
+		else if ( WIFEXITED(child_status) ) 
+		{
+			return_code=WEXITSTATUS(child_status);
+			termination_reason="exit code";
+		}	
+		slon_log(SLON_CONFIG, "slon: child terminated %s: %d; pid: %d, current worker pid: %d\n", termination_reason,return_code, pid, slon_worker_pid);
 
-		default:
-			break;
-	}
+
+		switch (watchdog_status)
+		{
+			case SLON_WATCHDOG_RESTART:
+				slon_log(SLON_CONFIG,"slon: restart of worker in 20 seconds\n");
+				sleep(20);
+				slon_worker_pid = fork();
+				if(slon_worker_pid==0) 
+				{
+					worker_restarted=1;
+					SlonMain();
+					exit(-1);
+				}
+				else if (slon_worker_pid < 0) 
+				{
+					slon_log(SLON_FATAL, "slon: failed to fork child: %d %s\n",
+							 errno,strerror(errno));
+					slon_exit(-1);
+					
+				}				
+				watchdog_status=SLON_WATCHDOG_NORMAL;
+				continue;
+				
+			case SLON_WATCHDOG_NORMAL:
+			case SLON_WATCHDOG_RETRY:
+				watchdog_status = SLON_WATCHDOG_RETRY;
+				if (child_status != 0)
+				{
+					slon_log(SLON_CONFIG, "slon: restart of worker in 10 seconds\n");
+					(void) sleep(10);
+				}
+				else
+				{
+					slon_log(SLON_CONFIG, "slon: restart of worker\n");
+				}
+				if (watchdog_status == SLON_WATCHDOG_RETRY)
+				{
+					slon_worker_pid=fork();
+					if(slon_worker_pid == 0)
+					{
+						worker_restarted=1;
+						SlonMain();
+						exit(-1);
+					}
+					else if (slon_worker_pid < 0) 
+					{
+						slon_log(SLON_FATAL, "slon: failed to fork child: %d %s\n",
+								 errno,strerror(errno));
+						slon_exit(-1);
+						
+					}
+					watchdog_status=SLON_WATCHDOG_NORMAL;
+					continue;
+				}
+				break;
+				
+			default:
+			  shutdown=1;
+				break;
+		} /*switch*/
+	}/*while*/
 
 	slon_log(SLON_INFO, "slon: done\n");
 
@@ -916,7 +994,6 @@ sighandler(int signo)
 	switch (signo)
 	{
 		case SIGALRM:
-			slon_log(SLON_INFO, "slon: child termination timeout - kill child\n");
 			kill(slon_worker_pid, SIGKILL);
 			break;
 
@@ -924,32 +1001,28 @@ sighandler(int signo)
 			break;
 
 		case SIGHUP:
-			slon_log(SLON_INFO, "slon: restart requested\n");
 			watchdog_status = SLON_WATCHDOG_RESTART;
 			slon_terminate_worker();
 			break;
 
 		case SIGUSR1:
-			slon_log(SLON_INFO, "slon: retry requested\n");
 			watchdog_status = SLON_WATCHDOG_RETRY;
 			slon_terminate_worker();
 			break;
 
 		case SIGINT:
 		case SIGTERM:
-			slon_log(SLON_INFO, "slon: shutdown requested\n");
 			watchdog_status = SLON_WATCHDOG_SHUTDOWN;
 			slon_terminate_worker();
 			break;
 
 		case SIGQUIT:
-			slon_log(SLON_INFO, "slon: shutdown now requested\n");
 			kill(slon_worker_pid, SIGKILL);
 			slon_exit(-1);
 			break;
 	}
 }
-#endif
+
 
 /* ---------- 
  * slon_terminate_worker 
@@ -958,21 +1031,10 @@ sighandler(int signo)
 void
 slon_terminate_worker()
 {
-#ifndef WIN32 /* does not support in windows. */
-	slon_log(SLON_INFO, "slon: notify worker process to shutdown\n");
+	(void) kill(slon_worker_pid, SIGKILL);
 
-	if (pipewrite(sched_wakeuppipe[1], "p", 1) != 1)
-	{
-		slon_log(SLON_FATAL, "main: write to worker pipe failed -(%d) %s\n", errno, strerror(errno));
-		(void) kill(slon_worker_pid, SIGKILL);
-		slon_exit(-1);
-	}
-        (void) close(sched_wakeuppipe[0]);
-	(void) close(sched_wakeuppipe[1]);
-	(void) alarm(20);
-#endif
 }
-
+#endif
 /* ---------- 
  * slon_exit 
  * ----------
@@ -996,6 +1058,25 @@ slon_exit(int code)
 	exit(code);
 }
 
+static sighandler_t install_signal_handler(int signo,  sighandler_t handler)
+{
+  
+#ifndef CYGWIN
+	struct sigaction act;
+	act.sa_handler = handler;
+	(void) sigemptyset(&act.sa_mask);
+	act.sa_flags = SA_NODEFER;
+
+
+	if(sigaction(signo, &act, NULL) < 0) 
+	{
+		return SIG_ERR;
+	}
+	return handler;
+#else
+	return signal(signo,handler);
+#endif
+}
 
 /*
  * Local Variables:
