@@ -1,5 +1,5 @@
 #!@@PERL@@
-# $Id: slonik_build_env.pl,v 1.3 2007-10-26 18:23:04 cbbrowne Exp $
+# 
 # Contributed by:
 # Joe Kalash
 # kalash@savicom.net
@@ -7,7 +7,7 @@
 # This script, given parameters concerning the database nodes,
 # generates output for "slon_tools.conf" consisting of:
 # - A set of add_node() calls to configure the cluster
-# - The arrays @KEYEDTABLES, @SERIALTABLES, and @SEQUENCES
+# - The arrays @KEYEDTABLES, and @SEQUENCES
 
 use DBI;
 use Getopt::Long;
@@ -92,11 +92,14 @@ if ( scalar(@tablesWithIndexes) >= 1 ) {
   print ");\n";
 }
 if ( scalar(@tablesWithoutIndexes) >= 1 ) {
-  print '@SERIALTABLES=(' . "\n";
+  my $tables = ''; 
   foreach my $table (sort @tablesWithoutIndexes) {
-    print "\t\"$table\",\n";
+	  if($tables ne '') {
+		  $tables.=',';
+	  }
+	  $tables.="\"$table\"";
   }
-  print ");\n";
+  die "The following tables had no unique index:" . $tables."\n";
 }
 if ( scalar(@sequences) >= 1 ) {
   print '@SEQUENCES=(' . "\n";
